﻿//【FT232R CBUS値 リセットツール】(C)jsdiy/2011~  http://jsdiy.web.fc2.com
//
//更新履歴
//20111030	ver1.00	初版

using System;
using FTD2XX_NET;

namespace Ft232rCbusReset
{
	//終了コード
	enum ExitCode
	{
		NormalTermination = 0,
		//
		FT232_OpenError = 10,
		FT232_Error,
		FT232_EepRomReadError,
		FT232_EepRomWriteError,
		//
		AnyReason = 255
	};

	//CBUSが取り得る値
	struct FtCbusOption
	{
		public static readonly string[] name =
		{
			"Tx Data Enable",	//0x00
			"Power On",	//0x01
			"Rx LED",	//0x02
			"Tx LED",	//0x03
			"Tx and Rx LED",	//0x04
			"Sleep",	//0x05
			"48MHz clock",	//0x06
			"24MHz clock",	//0x07
			"12MHz clock",	//0x08
			"6MHz clock",	//0x09
			"IO mode",	//0x0A
			"Bit-bang write strobe",	//0x0B
			"Bit-bang read strobe"	//0x0C
		};
		
		public static readonly byte[] defaultValue =
		{
			0x03,	//CBUS0
			0x02,	//CBUS1
			0x00,	//CBUS2
			0x01,	//CBUS3
			0x05	//CBUS4
		};
	}

	class Program
	{
		private static FTDI ftdev = null;
		
		static void Main(string[] args)
		{
			Console.WriteLine("【FT232R CBUS値 リセットツール】(C)jsdiy/2011~  http://jsdiy.web.fc2.com");
			
			//デバイスオープン
			ftdev = new FTDI();
			if (ftdev.OpenByIndex(0) != FTDI.FT_STATUS.FT_OK)
				AppClose("FT232 open error.", ExitCode.FT232_OpenError);
			Console.WriteLine("FT232 open");
			Console.WriteLine();

			//デバイス型番チェック
			uint devCount = 0;
			if (ftdev.GetNumberOfDevices(ref devCount) != FTDI.FT_STATUS.FT_OK || devCount == 0)
				AppClose("FT232 device error.", ExitCode.FT232_Error);
			FTDI.FT_DEVICE_INFO_NODE[] ftdevInfoList = new FTDI.FT_DEVICE_INFO_NODE[devCount];
			if (ftdev.GetDeviceList(ftdevInfoList) != FTDI.FT_STATUS.FT_OK)
				AppClose("FT232 device error.", ExitCode.FT232_Error);
			if (ftdevInfoList[0].Type != FTDI.FT_DEVICE.FT_DEVICE_232R)
				AppClose("FT232Rではありません。アプリケーションを終了します。", ExitCode.FT232_Error);

			//CBUSの現在の値を表示する
			//・全角／半角とも1文字と数えられるため、見出しの桁位置指定は画面を見ながら適当に調整する。
			FTDI.FT232R_EEPROM_STRUCTURE eepRomData = new FTDI.FT232R_EEPROM_STRUCTURE();
			if (ftdev.ReadFT232REEPROM(eepRomData) != FTDI.FT_STATUS.FT_OK)
				AppClose("FT232 EEPROM read error.", ExitCode.FT232_EepRomReadError);
			byte[] cbus = { eepRomData.Cbus0, eepRomData.Cbus1, eepRomData.Cbus2, eepRomData.Cbus3, eepRomData.Cbus4 };
			Console.WriteLine("{0,8}{1,-21}{2}", "", "現在の値", "デフォルト値");
			for (int i = 0; i < cbus.Length; i++)
				Console.WriteLine("CBUS-{0}: {1,-25}{2}", i, FtCbusOption.name[cbus[i]], FtCbusOption.name[FtCbusOption.defaultValue[i]]);
			Console.WriteLine();
			Console.WriteLine("デフォルト値に書き換えますか？ [1]または[0]キーを押してください。");
			Console.Write("[1]書き換える  [0]キャンセル  ");
			
			//処理選択のキー入力待ち
			//・ReadKey(true)…入力キー非表示。間違ったキーを入力しても見苦しくない。
			ConsoleKeyInfo keyInfo;
			do { keyInfo = Console.ReadKey(true); }
			while (keyInfo.KeyChar != '1' && keyInfo.KeyChar != '0');
			Console.WriteLine(keyInfo.KeyChar);

			//CBUSの値を書き換える
			Console.WriteLine();
			if (keyInfo.KeyChar == '1')
			{
				eepRomData.Cbus0 = FtCbusOption.defaultValue[0];
				eepRomData.Cbus1 = FtCbusOption.defaultValue[1];
				eepRomData.Cbus2 = FtCbusOption.defaultValue[2];
				eepRomData.Cbus3 = FtCbusOption.defaultValue[3];
				eepRomData.Cbus4 = FtCbusOption.defaultValue[4];
				
				Console.WriteLine("Wait...");
				if (ftdev.WriteFT232REEPROM(eepRomData) == FTDI.FT_STATUS.FT_OK)
					Console.WriteLine("CBUSの値を書き換えました。");
				else
					AppClose("書き換えに失敗しました。" + Environment.NewLine +
						"アプリケーションを再実行してCBUSの値を確認してください。", ExitCode.FT232_EepRomWriteError);
			}
			else
			{
				Console.WriteLine("処理をキャンセルしました。");
			}
			Console.WriteLine();
			
			//アプリ終了（デバイスクローズ）
			AppClose();
		}
		
		//アプリ終了
		private static void AppClose() { AppClose(null, ExitCode.NormalTermination); }
		private static void AppClose(string mes, ExitCode code)
		{
			//終了メッセージを表示する
			if (mes != null) Console.WriteLine(mes + Environment.NewLine);

			//FT232 クローズ
			if (ftdev != null && ftdev.IsOpen)
			{
				ftdev.Close();
				Console.WriteLine("FT232 close");
			}

			Console.WriteLine("Press any key.");
			Console.ReadKey();
			Environment.Exit((int)code);
		}
		
	}
}
